#include "fb.h"
#include "font_6x11.h"
#include "console.h"

#define CONSOLE_BUFFER_SIZE 4096
#define CONSOLE_BUFFER_MASK (CONSOLE_BUFFER_SIZE - 1)

static int cursor_x, cursor_y;
static int console_top = 0;
static int console_bottom = 0;
static int console_left = 0;
static int console_right = 0;
static char console_data[CONSOLE_BUFFER_SIZE];
static int console_display;
static int console_pos;

void console_clear()
{
  fb_fill(console_left, console_top,
	  console_right - console_left,
	  console_bottom - console_top,
	  CONSOLE_BGCOLOR);
}

void console_scroll(int anim)
{
  int i;

  if (anim) {
    for (i = 0; i < FONTHEIGHT; i++) {
      fb_move(console_left, console_top + 1,
	      console_right - console_left, console_bottom - console_top - 1,
	      console_left, console_top);
      fb_fill(console_left, console_bottom - 1,
	      console_right - console_left, 1, CONSOLE_BGCOLOR);
    }
  } else {
    fb_move(console_left, console_top + FONTHEIGHT,
	    console_right - console_left,
	    console_bottom - console_top - FONTHEIGHT,
	    console_left, console_top);
    fb_fill(console_left, console_bottom - FONTHEIGHT,
	    console_right - console_left, FONTHEIGHT, CONSOLE_BGCOLOR);
  }
}

static int console_drawchar(int c, int anim)
{
  if(cursor_x + FONTWIDTH > console_right || c == '\n') { /* new line */
    cursor_y += FONTHEIGHT;
    cursor_x = console_left;
  }

  if(cursor_y + FONTHEIGHT > console_bottom) { /* scroll up */
    cursor_y -= FONTHEIGHT;
    cursor_x = console_left;
    console_scroll(anim);
  }

  if(c == '\r')
    cursor_x = console_left;

  if(c != '\r' && c != '\n') { /* standard char */
    fb_draw_char(cursor_x, cursor_y, c, CONSOLE_FGCOLOR);
    cursor_x += FONTWIDTH;
  }

  return(c);
}

void console_init(void)
{
  console_pos = 0;
  console_data[0] = 0;
  console_display = 1;
}

void console_redraw(void)
{
  int i = 0;

  if (console_display) {
    console_clear();

    while (console_data[i] != 0) {
      console_drawchar(console_data[i], 0);
      i++;
      i &= CONSOLE_BUFFER_MASK;
    }
  }
}

void console_resize(int x, int y, int w, int h)
{
  console_top = y;
  console_bottom = y + h;
  console_left = x;
  console_right = x + w;
  cursor_x = console_left;
  cursor_y = console_top;
  console_redraw();
}

void console_hide(void)
{
  console_display = 0;
}

void console_show(void)
{
  console_display = 1;
  console_redraw();
}

int console_visible(void)
{
  return console_display;
}

int putchar(int c)
{
  if (c == 0) return c;

  console_data[console_pos] = c;
  console_pos ++;
  console_pos &= CONSOLE_BUFFER_MASK;
  console_data[console_pos] = 0;

  if (console_display)
    console_drawchar(c, 1);

  return c;
}
