/* Embedded OS Loader
 *
 *   Copyright 2005 Vivien Chappelier <vivien.chappelier@free.fr>
 *
 * GPIO programming
 *
 */
#include "stdlib.h"
#include "gpio.h"

#define GPIO_BASE 0xfffbc000
#define GPIO_BANK      0x800

#define GPIO_IN		0x00
#define GPIO_OUT	0x04
#define GPIO_DIR	0x08

#define GPIO_REG(gpio, reg) REG32(GPIO_BASE + GPIO_BANK * (gpio >> 5) + (reg))

/* set a GPIO output value */
void gpio_out(int gpio, int value)
{
	if(value)
		GPIO_REG(gpio, GPIO_OUT) |= 1 << (gpio & 31);
	else
		GPIO_REG(gpio, GPIO_OUT) &= ~(1 << (gpio & 31));
}

/* read a GPIO value */
int gpio_in(int gpio)
{
	return((GPIO_REG(gpio, GPIO_IN) & (1 << (gpio & 31))) != 0);
}

/* set a GPIO direction */
void gpio_dir(int gpio, int input)
{
	if(input)
		GPIO_REG(gpio, GPIO_DIR) |= 1 << (gpio & 31);
	else
		GPIO_REG(gpio, GPIO_DIR) &= ~(1 << (gpio & 31));
}
