/* Embedded OS Loader
 *
 *   Copyright 2005 Vivien Chappelier <vivien.chappelier@free.fr>
 *
 * LED programming
 *
 */
#include "stdlib.h"
#include "gpio.h"
#include "mux.h"
#include "led.h"
#include "model.h"

#define LED_BASE   0xfffba800
#define LED_CTRL   0x00
#define LED_ENABLE 0x01
#	define LED_ON         (1 << 7)
#	define LED_BLINK      (1 << 6)
#	define LED_ONTIME(x)  ((x) & 0x7)
#	define LED_OFFTIME(x) (((x) & 0x7) << 3)

#define LED_REG(reg) REG8(LED_BASE + (reg))

/* initialize the led */
void led_init(void)
{
	/* route power to the tricolor led */
	ioconf_set(10, 0xd0);
	gpio_dir(GPIO_LED_BLUE, 0); /* out */
	iomode_set(29);
	
	LED_REG(LED_CTRL) = LED_ON;
	LED_REG(LED_ENABLE) = 1;
}

/* set the 'blue', 'green' and 'red' components of the tricolor led */
void led_set(int led)
{
	if(led & LED_BLUE)
		gpio_out(GPIO_LED_BLUE, 1);
	else
		gpio_out(GPIO_LED_BLUE, 0);
	
	if(led & (LED_RED | LED_GREEN)) {
		if(led & LED_GREEN) {
			gpio_dir(GPIO_LED_GREEN, 1);
		} else {
			gpio_dir(GPIO_LED_GREEN, 0);
			gpio_out(GPIO_LED_GREEN, 0);
		}
		
		if(led & LED_RED) {
			gpio_dir(GPIO_LED_RED, 1);
		} else {
			gpio_dir(GPIO_LED_RED, 0);
			gpio_out(GPIO_LED_RED, 0);
		}
	} else {
		gpio_dir(GPIO_LED_GREEN, 0);
		gpio_out(GPIO_LED_GREEN, 0);
		gpio_dir(GPIO_LED_RED, 0);
		gpio_out(GPIO_LED_RED, 0);
	}
}
