/* MMC/SD card controller I/O */
#ifndef __MMC_H
#define __MMC_H

#define     OMAP730_MMC_BASE  0xfffb7800

/* registers are 16-bit wide */
#define	MMC_REG_CMD	0x00
#define	MMC_REG_ARGL	0x02
#define	MMC_REG_ARGH	0x04
#define	MMC_REG_CON	0x06
#define	MMC_REG_STAT	0x08 /**/
#define	MMC_REG_IE	0x0a
#define	MMC_REG_CTO	0x0c
#define	MMC_REG_DTO	0x0e
#define	MMC_REG_DATA	0x10 /**/
#define	MMC_REG_BLEN	0x12 /**/
#define	MMC_REG_NBLK	0x14 /**/
#define	MMC_REG_BUF	0x16 /**/
#define	MMC_REG_REV	0x1e
#define	MMC_REG_RSP0	0x20 /**/
#define	MMC_REG_RSP1	0x22 /**/
#define	MMC_REG_RSP2	0x24 /**/
#define	MMC_REG_RSP3	0x26 /**/
#define	MMC_REG_RSP4	0x28 /**/
#define	MMC_REG_RSP5	0x2a /**/
#define	MMC_REG_RSP6	0x2c /**/
#define	MMC_REG_RSP7	0x2e /**/
#define	MMC_REG_IOSR	0x30
#define	MMC_REG_SYSC	0x32
#define	MMC_REG_SYSS	0x34

#define MMC_READ(reg) 	REG16(OMAP730_MMC_BASE + MMC_REG_##reg)
#define MMC_WRITE(reg, val) do { REG16(OMAP730_MMC_BASE + MMC_REG_##reg) = val; } while(0);

/* status register */
#define	MMC_STAT_CARD_ERR	(1 << 14)
#define	MMC_STAT_CARD_IRQ	(1 << 13)
#define	MMC_STAT_OCR_BUSY	(1 << 12)
#define	MMC_STAT_A_EMPTY	(1 << 11)
#define	MMC_STAT_A_FULL		(1 << 10)
#define	MMC_STAT_CMD_CRC	(1 <<  8)
#define	MMC_STAT_CMD_TOUT	(1 <<  7)
#define	MMC_STAT_DATA_CRC	(1 <<  6)
#define	MMC_STAT_DATA_TOUT	(1 <<  5)
#define	MMC_STAT_END_BUSY	(1 <<  4)
#define	MMC_STAT_END_OF_DATA	(1 <<  3)
#define	MMC_STAT_CARD_BUSY	(1 <<  2)
#define	MMC_STAT_END_OF_CMD	(1 <<  0)

/* command types */
#define MMC_CMDTYPE_BC	 0
#define MMC_CMDTYPE_BCR	 1
#define MMC_CMDTYPE_AC	 2
#define MMC_CMDTYPE_ADTC 3

/* response types */
#define MMC_RSP_NONE	(0 << 0)
#define MMC_RSP_SHORT	(1 << 0)
#define MMC_RSP_LONG	(2 << 0)
#define MMC_RSP_MASK	(3 << 0)
#define MMC_RSP_BUSY	(1 << 3)		/* card may send busy */

/* other command flags */
#define MMC_DIR_WRITE (0 << 15)                 /* write to MMC */
#define MMC_DIR_READ  (1 << 15)                 /* read from MMC */

/* Standard MMC commands (3.1)           type  argument     response */
#define	MMC_OP_GO_IDLE_STATE         0   /* bc                          */
#define MMC_OP_SEND_OP_COND          1   /* bcr  [31:0] OCR         R3  */
#define MMC_OP_ALL_SEND_CID          2   /* bcr                     R2  */
#define MMC_OP_SET_RELATIVE_ADDR     3   /* ac   [31:16] RCA        R1  */
#define MMC_OP_SET_DSR               4   /* bc   [31:16] RCA            */
#define MMC_OP_SELECT_CARD           7   /* ac   [31:16] RCA        R1  */
#define MMC_OP_SEND_CSD              9   /* ac   [31:16] RCA        R2  */
#define MMC_OP_SEND_CID             10   /* ac   [31:16] RCA        R2  */
#define MMC_OP_READ_DAT_UNTIL_STOP  11   /* adtc [31:0] dadr        R1  */
#define MMC_OP_STOP_TRANSMISSION    12   /* ac                      R1b */
#define MMC_OP_SEND_STATUS          13   /* ac   [31:16] RCA        R1  */
#define MMC_OP_GO_INACTIVE_STATE    15   /* ac   [31:16] RCA            */
#define MMC_OP_SET_BLOCKLEN         16   /* ac   [31:0] block len   R1  */
#define MMC_OP_READ_SINGLE_BLOCK    17   /* adtc [31:0] data addr   R1  */
#define MMC_OP_READ_MULTIPLE_BLOCK  18   /* adtc [31:0] data addr   R1  */
#define MMC_OP_WRITE_DAT_UNTIL_STOP 20   /* adtc [31:0] data addr   R1  */
#define MMC_OP_SET_BLOCK_COUNT      23   /* adtc [31:0] data addr   R1  */
#define MMC_OP_WRITE_BLOCK          24   /* adtc [31:0] data addr   R1  */
#define MMC_OP_WRITE_MULTIPLE_BLOCK 25   /* adtc                    R1  */
#define MMC_OP_PROGRAM_CID          26   /* adtc                    R1  */
#define MMC_OP_PROGRAM_CSD          27   /* adtc                    R1  */
#define MMC_OP_SET_WRITE_PROT       28   /* ac   [31:0] data addr   R1b */
#define MMC_OP_CLR_WRITE_PROT       29   /* ac   [31:0] data addr   R1b */
#define MMC_OP_SEND_WRITE_PROT      30   /* adtc [31:0] wpdata addr R1  */
#define MMC_OP_ERASE_GROUP_START    35   /* ac   [31:0] data addr   R1  */
#define MMC_OP_ERASE_GROUP_END      36   /* ac   [31:0] data addr   R1  */
#define MMC_OP_ERASE                37   /* ac                      R1b */
#define MMC_OP_FAST_IO              39   /* ac   <Complex>          R4  */
#define MMC_OP_GO_IRQ_STATE         40   /* bcr                     R5  */
#define MMC_OP_LOCK_UNLOCK          42   /* adtc                    R1b */
#define MMC_OP_APP_CMD              55   /* ac   [31:16] RCA        R1  */
#define MMC_OP_GEN_CMD              56   /* adtc [0] RD/WR          R1b */
#define SD_APP_SET_BUS_WIDTH         6   /* ac   [1:0] bus width    R1  */
#define SD_APP_OP_COND              41   /* bcr  [31:0] OCR         R3  */
#define SD_APP_SEND_SCR             51   /* adtc                    R1  */

#define MMC_CMDBIT_OPEN_DRAIN 6
#define MMC_CMDBIT_RESET      7

#define MMC_MAKE_CMD(op, resp, type) ((op) | ((resp) << 8) | ((type) << 12))

#define MMC_CMD_READ_SINGLE_BLOCK   (MMC_MAKE_CMD(MMC_OP_READ_SINGLE_BLOCK,    \
						  MMC_RSP_SHORT,               \
						  MMC_CMDTYPE_ADTC) |          \
				     MMC_DIR_READ)
#define MMC_CMD_READ_MULTIPLE_BLOCK (MMC_MAKE_CMD(MMC_OP_READ_MULTIPLE_BLOCK,  \
                                                  MMC_RSP_SHORT,               \
						  MMC_CMDTYPE_ADTC) |          \
                                     MMC_DIR_READ)
#define MMC_CMD_STOP_TRANSMISSION   (MMC_MAKE_CMD(MMC_OP_STOP_TRANSMISSION,    \
                                                  MMC_RSP_SHORT | MMC_RSP_BUSY,\
                                                  MMC_CMDTYPE_AC))
#define MMC_CMD_SEND_CID            (MMC_MAKE_CMD(MMC_OP_ALL_SEND_CID,         \
                                                  MMC_RSP_LONG,                \
						  MMC_CMDTYPE_BCR) |           \
				     (1 << MMC_CMDBIT_OPEN_DRAIN))
#define MMC_CMD_RESET               (1 << MMC_CMDBIT_RESET)
#define MMC_CMD_GO_IDLE             MMC_OP_GO_IDLE_STATE
#define SD_CMD_APP_OP_COND          (MMC_MAKE_CMD(SD_APP_OP_COND,              \
                                                  MMC_RSP_SHORT,               \
						  MMC_CMDTYPE_BCR) |           \
				     (1 << MMC_CMDBIT_OPEN_DRAIN))
#define MMC_CMD_SEND_OP_COND        (MMC_MAKE_CMD(MMC_OP_SEND_OP_COND,         \
                                                  MMC_RSP_SHORT,               \
						  MMC_CMDTYPE_BCR) |           \
				     (1 << MMC_CMDBIT_OPEN_DRAIN))
#define MMC_CMD_APP_CMD             (MMC_MAKE_CMD(MMC_OP_APP_CMD,              \
                                                  MMC_RSP_SHORT,               \
						  MMC_CMDTYPE_AC))
#define MMC_CMD_SET_RELATIVE_ADDR   (MMC_MAKE_CMD(MMC_OP_SET_RELATIVE_ADDR,    \
                                                  MMC_RSP_SHORT,               \
                                                  MMC_CMDTYPE_AC))

#define MMC_RESET_TIMEOUT 100000000

#define SECTOR_SHIFT 9
#define SECTOR_SIZE (1 << SECTOR_SHIFT)
#define SECTOR_MASK (SECTOR_SIZE - 1)

/* reset the MMC controller */
int mmc_reset();

/* read data from the MMC */
int mmc_read(int offset, void *buffer, size_t length);

/* read the MMC unique id */
int mmc_get_cid(char cid[16]);

/* print the MMC unique id */
void print_cid(char cid[16]);

#endif
