#include "model.h"
#include "gpio.h"
#include "stdlib.h"
#include "fb.h"

#define OMAP_DIE_ID_0		0xfffe1800
#define OMAP_DIE_ID_1		0xfffe1804
#define OMAP_PRODUCTION_ID_0	0xfffe2000
#define OMAP_PRODUCTION_ID_1	0xfffe2004
#define OMAP32_ID_0		0xfffed400
#define OMAP32_ID_1		0xfffed404

/* omap identifiers */
#define TYPHOON_ID0 0x03320300
#define TYPHOON_ID1 0x1b55f02f
#define TORNADO_ID0 0x03320500
#define TORNADO_ID1 0x1b62c02f

/* get the phone model (by reading the OMAP version) */
int get_model(void)
{
  unsigned int omap32_id_0 = REG32(OMAP32_ID_0);
  unsigned int omap32_id_1 = REG32(OMAP32_ID_1);

  if (omap32_id_0 == TYPHOON_ID0 &&
      omap32_id_1 == TYPHOON_ID1)
    return MODEL_TYPHOON;

  if (omap32_id_0 == TORNADO_ID0 &&
      omap32_id_1 == TORNADO_ID1)
    return MODEL_TORNADO;

  return MODEL_UNKNOWN;
}

/* get the panel identifier */
int get_lcd_id(void)
{
  gpio_dir(GPIO_PANEL_ID0, 1);
  gpio_dir(GPIO_PANEL_ID1, 1);

  return (gpio_in(GPIO_PANEL_ID1) << 1) | gpio_in(GPIO_PANEL_ID0);
}

/* unknown model, dump relevant info and stop */
void panic_unknown_model(void)
{
    printb("omap_id_0 = %x\n", REG32(OMAP32_ID_0));
    printb("omap_id_1 = %x\n", REG32(OMAP32_ID_1));
    printb("lcd_id = %x\n", get_lcd_id());
    dump_fb_timing_info();
    printb("please report above infos\n");
    while(1);
}

/* print the model and LCD type */
void print_model_info(void)
{
  switch (get_model()) {
  case MODEL_TYPHOON:
    printb("Typhoon, ");
    switch (get_lcd_id()) {
    case TYPHOON_LCD_TOPPOLY:
      printb("Toppoly LCD\n");
      break;
    case TYPHOON_LCD_HUNDAI:
      printb("Hundai LCD\n");
      break;
    case TYPHOON_LCD_SONY:
      printb("Sony LCD\n");
      break;
    default:
      printb("unknown LCD\n");
      panic_unknown_model();
      break;
    }
    break;
  case MODEL_TORNADO:
    printb("Tornado, ");
    switch (get_lcd_id()) {
    case TORNADO_LCD_SONY:
      printb("Sony LCD\n");
      break;
    case TORNADO_LCD_TOPPOLY:
      printb("Toppoly LCD\n");
      break;
    default:
      printb("unknown LCD\n");
      panic_unknown_model();
      break;
    }
    break;
  default:
    printb("Unknown model\n");
    panic_unknown_model();
    break;
  }
}
