/* Embedded OS Loader
 *
 *   Copyright 2005 Vivien Chappelier <vivien.chappelier@free.fr>
 *
 * MUX programming
 *
 */
#include "stdlib.h"
#include "mux.h"

#define IOMODE_BASE	0xfffe1010
#define IOCONF_BASE	0xfffe1070

#define IOMODE_REG(reg) REG32(IOMODE_BASE + 4 * (reg))
#define IOCONF_REG(reg) REG32(IOCONF_BASE + 4 * (reg))

/* set MUX configuration register */
void ioconf_set(int reg, int mask)
{
	int imask = mask;

	if(!(mask & 0x0000000f)) imask |= 0x0000000f;
	if(!(mask & 0x000000f0)) imask |= 0x000000f0;
	if(!(mask & 0x00000f00)) imask |= 0x00000f00;
	if(!(mask & 0x0000f000)) imask |= 0x0000f000;
	if(!(mask & 0x000f0000)) imask |= 0x000f0000;
	if(!(mask & 0x00f00000)) imask |= 0x00f00000;
	if(!(mask & 0x0f000000)) imask |= 0x0f000000;
	if(!(mask & 0xf0000000)) imask |= 0xf0000000;

	IOCONF_REG(reg) &= imask;
	IOCONF_REG(reg) |= mask;
}

/* set MUX mode register bit */
void iomode_set(int bit)
{
	IOMODE_REG(0) |= (1 << bit);
}

/* clear MUX mode register bit */
void iomode_clear(int bit)
{
	IOMODE_REG(0) &= ~(1 << bit);
}

int iomode_get(void)
{
	return(IOMODE_REG(0));
}
