/* Embedded OS Loader
 *
 *   Copyright 2005 Vivien Chappelier <vivien.chappelier@free.fr>
 *
 * pin muxing configuration
 *
 */
#include "stdlib.h"
#include "mux.h"
#include "gpio.h"
#include "model.h"

#define GPIO_USB_ENABLE            33 /* USB enable */
#define GPIO_VIBRATOR              34 /* Vibrator */
#define GPIO_DM                    35 /* USB D- */
#define GPIO_DP                    36 /* USB D+ */

void gsm_init(void)
{
	gpio_dir(GPIO_USB_ENABLE, 1); /* input */

	ioconf_set(2, 0x00c00000);

	gpio_dir(31, 1); /* input */
	gpio_dir(32, 1); /* input */
	gpio_dir(35, 0); /* output */
	gpio_dir(36, 1); /* input */

	ioconf_set(3, 0x00000044);
}

/* configure pin muxing */
void pin_init()
{
	/* enable USB */
	do {
		gpio_dir(GPIO_USB_ENABLE, 0); /* out */
		gpio_out(GPIO_USB_ENABLE, 0); /* low */
	} while(gpio_in(GPIO_USB_ENABLE));

	gpio_out(73, 0);
	gpio_dir(GPIO_DM, 1); /* in */

	/* get uart control from GSM */
	ioconf_set(3, 0x000000cc);
	gpio_dir(GPIO_DP, 1); /* in */
	ioconf_set(2, 0x00100000); /* AA17_730_USB_DM, pull up/down disable */
	udelay(100000);
	ioconf_set(2, 0x01000000); /* W16_730_USB_PU_EN, pull up/down disable */
	iomode_set(25);

	/* OTG muxing */
	iomode_clear(2);
	iomode_clear(3);
	iomode_set(15);
	iomode_set(23);
	iomode_set(26);
	iomode_set(25);
	iomode_clear(24);
	iomode_clear(10);
	iomode_clear(11);
}

